@@ -29,6 +29,10 @@
 -- This is a one time use program to add a bunch of features to the PERSON class
 --
 -- $Log: add_class.c,v $
+-- Revision 1.22  2008/08/31 18:25:40  jsedwards
+-- Change for new parameters to nwos_initialize_objectify without key stuff
+-- and eliminate calle to nwos_set_root_object because initialize does that now.
+--
 -- Revision 1.21  2008/03/28 11:33:50  jsedwards
 -- Add test to verify that class doesn't already exist!!
 --
@@ -226,7 +230,6 @@ void add_feature(C_struct_Class_Definition* class_obj_ptr, ObjRef* type, char* n
 
 int main(int argc, char* argv[])
 {
-    ObjRef root_ref;
     ObjRef class_ref;
     uint8 kludge[FILE_BLOCK_SIZE];
     C_struct_Class_Definition* class_obj_ptr = (C_struct_Class_Definition*)kludge;
@@ -234,14 +237,8 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
-    nwos_initialize_objectify(NULL, 0, 0, PUBLIC, NULL);
+    nwos_initialize_objectify(PUBLIC, NULL);
 
-    root_ref.id[0] = 0;
-    root_ref.id[1] = 0;
-    root_ref.id[2] = 0;
-    root_ref.id[3] = 1;
-
-    nwos_set_root_object(&root_ref);
 
 #define NEW_CLASS_NAME "SHA512SUM"
 
