@@ -27,6 +27,20 @@
 --
 --
 -- $Log: disc_list.c,v $
+-- Revision 1.31  2008/09/01 03:13:41  jsedwards
+-- Change for new nwos_initialize_objectify calling convention (doesn't pass
+-- back root_object_reference anymore) and removed call to nwos_set_root_object
+-- because initialize calls it now.
+--
+-- Revision 1.30  2008/08/31 21:53:51  jsedwards
+-- Added an assert around calls to nwos_read_variable_sized_object_from_disk
+-- and nwos_read_object_from_disk because now they return false when they fail
+-- instead of asserting themselves.
+--
+-- Revision 1.29  2008/08/30 12:46:08  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
 -- Revision 1.28  2008/08/03 17:31:34  jsedwards
 -- Add --ignore-empty-directories and --ignore-empty-files options.
 --
@@ -180,11 +194,11 @@ void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
     int i;
     ObjRef object_class;
 
-    nwos_read_object_from_disk(new_assoc_ref, &new_assoc_obj, sizeof(new_assoc_obj));
+    assert(nwos_read_object_from_disk(new_assoc_ref, &new_assoc_obj, sizeof(new_assoc_obj)));
 
     assert(is_void_reference(&new_assoc_obj.header.object.prev_version));
 
-    nwos_read_variable_sized_object_from_disk(&new_assoc_obj.path, kludge, sizeof(kludge), &get_path_object_size);
+    assert(nwos_read_variable_sized_object_from_disk(&new_assoc_obj.path, kludge, sizeof(kludge), &get_path_object_size));
 
     ref_list = nwos_malloc_reference_list(&ptr_path_obj->header.object.references);
 
@@ -199,7 +213,7 @@ void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
 	    nwos_get_object_class(&ref_list->references[i], &object_class);
 	    if (is_same_object(&object_class, &new_assoc_obj.header.common.class_definition))
 	    {
-		nwos_read_object_from_disk(&ref_list->references[i], &old_assoc_obj, sizeof(old_assoc_obj));
+		assert(nwos_read_object_from_disk(&ref_list->references[i], &old_assoc_obj, sizeof(old_assoc_obj)));
 
 		printf("Old association: %08x  file: %08x  next_version: %08x\n",
 		       nwos_ref_to_word(&ref_list->references[i]),
@@ -230,10 +244,10 @@ void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
 		   nwos_ref_to_word(&old_assoc_obj.header.common.id),
 		   nwos_ref_to_word(&new_assoc_obj.header.common.id));
 
-	    nwos_read_object_from_disk(&old_assoc_obj.header.common.id, &assoc_obj, sizeof(assoc_obj));
+	    assert(nwos_read_object_from_disk(&old_assoc_obj.header.common.id, &assoc_obj, sizeof(assoc_obj)));
 	    assert(memcmp(&assoc_obj, &old_assoc_obj, sizeof(assoc_obj)) == 0);
 
-	    nwos_read_object_from_disk(&new_assoc_obj.header.common.id, &assoc_obj, sizeof(assoc_obj));
+	    assert(nwos_read_object_from_disk(&new_assoc_obj.header.common.id, &assoc_obj, sizeof(assoc_obj)));
 	    assert(memcmp(&assoc_obj, &new_assoc_obj, sizeof(assoc_obj)) == 0);
 	}
 
@@ -243,12 +257,7 @@ void check_for_previous_version_and_link(ObjRef* new_assoc_ref)
 
 int main(int argc, char* argv[])
 {
-    ObjRef root_object_ref;
     ObjRef public_ref;
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
     ObjRef ref;
     ObjCreateResult result;
     int i;
@@ -323,16 +332,6 @@ int main(int argc, char* argv[])
 
     num_files &= DISC_LIST_NUM_FILES_MASK;
 
-    /*********************************/
-    /* Get the pass phrase from user */
-    /*********************************/
-
-    printf("\n");
-
-    nwos_get_key_from_password(big_key, sizeof(big_key));
-
-    printf("\n");
-
 
     /*******************/
     /* Get the disc id */
@@ -380,14 +379,7 @@ int main(int argc, char* argv[])
 
     if (!ignore_public)
     {
-	nwos_initialize_objectify(NULL, 0, 0, PUBLIC, NULL);
-
-	root_object_ref.id[0] = 0;
-	root_object_ref.id[1] = 0;
-	root_object_ref.id[2] = 0;
-	root_object_ref.id[3] = 1;
-
-	nwos_set_root_object(&root_object_ref);
+	nwos_initialize_objectify(PUBLIC, NULL);
 
 	for (i = 0; i < num_files; i++)
 	{
@@ -442,16 +434,10 @@ int main(int argc, char* argv[])
     /* Now open Objectify in private, read-write mode to do the real thing */
     /***********************************************************************/
 
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
 
     nwos_log_arguments(argc, argv);
 
-    nwos_initialize_objectify(bf_key, linear, serial, READ_WRITE, DEFAULT_FILE);
-
-    nwos_set_root_object(&root_object_ref);
+    nwos_initialize_objectify(READ_WRITE, DEFAULT_FILE);
 
     nwos_set_security_level(Security_Very_Low);
 
