@@ -27,6 +27,71 @@
 --
 --
 -- $Log: disk_io.c,v $
+-- Revision 1.108  2008/11/01 10:22:09  jsedwards
+-- Missed one change of RESERVED_PUBLIC_BLOCKS to MAXIMUM_PUBLIC_REFERENCE in
+-- previous check in.
+--
+-- Revision 1.107  2008/10/31 13:43:04  jsedwards
+-- Moved nwos_reference_type and is_valid_private_reference functions from
+-- objectify.c and renamed the latter to nwos_is_private_reference.  Added new
+-- nwos_is_public_reference and nwos_is_temporary_reference functions.  Changed
+-- all references to RESERVED_PUBLIC_BLOCKS to MAXIMUM_PUBLIC_REFERENCE or
+-- MINIMUM_PRIVATE_REFERENCE and MAXIMUM_VALID_PRIVATE_REFERENCE to
+-- MAXIMUM_PRIVATE_REFERENCE.
+--
+-- Revision 1.106  2008/10/28 10:23:52  jsedwards
+-- Changed skip_forward and skip_backwards parameters to unsigned.
+--
+-- Revision 1.105  2008/10/27 14:17:36  jsedwards
+-- Changed so that if backup directory is an empty string (as it is if the
+-- environment variable is defined but empty) it does not make a backup.
+--
+-- Revision 1.104  2008/10/27 04:07:42  jsedwards
+-- Remove new line character from log message.
+--
+-- Revision 1.103  2008/10/27 04:06:39  jsedwards
+-- Changed nwos_check_blocks_available to search backwards for a new chunk
+-- when it reaches the end of the private chunk space.
+--
+-- Revision 1.102  2008/10/27 01:48:18  jsedwards
+-- Changed 'index' in Bit_Map_Cache_Entry structure to 'chunk_index' so that it
+-- is a chunk index instead an index into the chunk_info table.  This was
+-- necessary because if the chunk_info table has a new entry inserted the bit
+-- map cache will have the wrong indexes.
+--
+-- Revision 1.101  2008/10/27 00:48:21  jsedwards
+-- Fixed spacing that was screwed up when the chunk_index_to_info_index was
+-- moved in the previous revision.  NO code changes.
+--
+-- Revision 1.100  2008/10/27 00:20:38  jsedwards
+-- Moved chunk_index_to_info_index function so that it appears before the
+-- write_bit_map function that will need it after the next change.
+--
+-- Revision 1.99  2008/10/27 00:09:20  jsedwards
+-- Doh!  Put the Bug # back into the log entry for revision 1.96.  This version
+-- is the same as revision 1.97 with only the log entries for 1.98 and 1.99
+-- added.  NO code changes!
+--
+-- Revision 1.98  2008/10/26 23:49:01  jsedwards
+-- Removed line added to log in last revision, I had a brain cramp and added it
+-- but then realized that change did not fix the bug.  This version is the same
+-- as version 1.96 except for the logs for 1.97 and 1.98.  NO code changes.
+--
+-- Revision 1.97  2008/10/26 18:07:53  jsedwards
+-- Added bug number to previous log entry (Revision 1.96).  NO code changes.
+--
+-- Revision 1.96  2008/10/26 17:11:10  jsedwards
+-- Fix Bug #2184020 - Assert when adding files and chunk is near upper limit.
+-- Fixed test to see if it is going to allocate a block beyond the maximum
+-- valid private reference.
+--
+-- Revision 1.95  2008/09/25 13:46:41  jsedwards
+-- Fix Bug #2126403 - changed to not process chunk_info table when using a
+-- compressed file.
+--
+-- Revision 1.94  2008/09/19 13:11:55  jsedwards
+-- Added a warning if any chunks are located above MAXIMUM_VALID_PRIVATE_REFERENCE.
+--
 -- Revision 1.93  2008/09/01 18:08:19  jsedwards
 -- Added chunk_info_reverse_index to get from chunk number back to an index
 -- into the chunk_info table.
@@ -437,7 +502,7 @@ static ObjRef last_ref;
 typedef struct {
   bool8 in_use;
   bool8 dirty;
-  int index;    /* index into chunk_info table */
+  int chunk_index;    /* index into chunk (NOT chunk_info index) */
   int age;
   uint8* map;
 } Bit_Map_Cache_Entry;
@@ -450,11 +515,54 @@ static void write_bit_map(Bit_Map_Cache_Entry* entry);
 
 #ifndef PUBLIC_MODE
 static uint32 chunk_density_target;
-static int32  chunk_skip_backward;
-static int32  chunk_skip_forward;
+static uint32 chunk_skip_backward;
+static uint32 chunk_skip_forward;
 #endif
 
 
+/**********************/
+/* Reference testing. */
+/**********************/
+
+bool nwos_is_public_reference(ObjRef* ref)
+{
+    return nwos_ref_to_word(ref) <= MAXIMUM_PUBLIC_REFERENCE;
+}
+
+
+bool nwos_is_private_reference(ObjRef* ref)
+{
+    return MINIMUM_PRIVATE_REFERENCE <= nwos_ref_to_word(ref) && nwos_ref_to_word(ref) <= MAXIMUM_PRIVATE_REFERENCE;
+}
+
+
+bool nwos_is_temporary_reference(ObjRef* ref)
+{
+    return MINIMUM_TEMPORARY_REFERENCE <= nwos_ref_to_word(ref) && nwos_ref_to_word(ref) <= MAXIMUM_TEMPORARY_REFERENCE;
+}
+
+
+Reference_Type nwos_reference_type(ObjRef* ref)
+{
+    if (nwos_is_public_reference(ref))
+    {
+	return Public_Reference;
+    }
+
+    if (nwos_is_private_reference(ref))
+    {
+	return Private_Reference;
+    }
+
+    if (nwos_is_temporary_reference(ref))
+    {
+	return Temporary_Reference;
+    }
+
+    return Unknown_Reference;     /* this should never happen */
+}
+
+
 /*************************/
 /* Block estimate stuff. */
 /*************************/
@@ -717,7 +825,7 @@ static void update_chunk_info_reverse_index()
 void nwos_initialize_disk_io(AccessType type, const char* path)
 {
     size_t chunk_info_size_in_bytes;
-    char log_msg[128];
+    char log_msg[256];
     uint32 used_blocks;
     int i;
     int j;
@@ -903,103 +1011,116 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
 	nwos_total_private_chunks = nwos_total_private_blocks / BLOCKS_IN_CHUNK;
 
-	if (lseek(private_file_desc, FILE_BLOCK_SIZE, SEEK_SET) < 0)
-	{
-	    perror("chunk index");
-	    exit(1);
-	}
-
-	chunk_info_size_in_bytes = nwos_total_private_chunks * sizeof(Chunk_Info);
-	chunk_info = nwos_malloc(chunk_info_size_in_bytes);
-
-	if (read(private_file_desc, chunk_info, chunk_info_size_in_bytes) != chunk_info_size_in_bytes)
-	{
-	    snprintf(log_msg, sizeof(log_msg), "reading chunk info from: %s", nwos_private_path);
-	    perror(log_msg);
-	    exit(1);
-	}
-
-	used_blocks = 0;
-	for (i = 0; i < (int)nwos_total_private_chunks; i++)
-	{
-#ifndef WORDS_BIGENDIAN
-	    chunk_info[i].ref = byteswap_uint32(chunk_info[i].ref);
-	    chunk_info[i].used = byteswap_uint16(chunk_info[i].used);
-	    chunk_info[i].index = byteswap_uint16(chunk_info[i].index);
-#endif
-	    used_blocks += chunk_info[i].used;
-	}
-
-	if (used_blocks != nwos_used_private_blocks)
-	{
-	    snprintf(log_msg, sizeof(log_msg), 
-		     "Warning: calculated sum of used blocks (%u) doesn't match stored (%u)",
-		     used_blocks, nwos_used_private_blocks);
-	    nwos_log(log_msg);
-	    fprintf(stderr, "%s\n", log_msg);
-	}
-
-	chunk_info_modified = false;
-
-	update_chunk_info_reverse_index();
-
-
-	/* now create the chunk_used_index */
-
-	chunk_info_size_in_bytes = nwos_total_private_chunks * sizeof(uint16);
-	chunk_used_index = nwos_malloc(chunk_info_size_in_bytes);
-
-	/* insert them into a heap to do a heap sort */
-	for (i = 0; i < (int)nwos_used_private_chunks; i++)
-	{
-	    j = i + 1;
-	    while (j / 2 > 0 && chunk_info[chunk_used_index[(j / 2) - 1]].used < chunk_info[i].used)
-	    {
-		/* parent is smaller, move it lower in the tree */
-		chunk_used_index[j - 1] = chunk_used_index[(j / 2) - 1];
-		j = j / 2;
-	    }
-
-	    chunk_used_index[j - 1] = i;
-	}
-
-	for (i = (int)nwos_used_private_chunks; i > 1; i--)
-	{
-	    /* save the index at the top of the tree (smallest) */
-	    save = chunk_used_index[i - 1];
-
-	    /* move the root of the the (largest) to the top */
-	    chunk_used_index[i - 1] = chunk_used_index[0];
-
-	    j = 1;
-	    while (j < (i / 2) - 1)
-	    {
-		k = j * 2;
-
-		/* if there is a right child */
-		if (k < i - 1)
-		{
-		    left = chunk_info[chunk_used_index[k - 1]].used;
-		    right = chunk_info[chunk_used_index[k + 1 - 1]].used;
-
-		    if (left < right)
-		    {
-			k++;
-		    }
-		}
-
-		if (chunk_info[save].used >= chunk_info[chunk_used_index[k-1]].used) break;
-
-		chunk_used_index[j - 1] = chunk_used_index[k - 1];
-
-		j = k;
-	    }
-
-	    chunk_used_index[j - 1] = save;
-	}
+	/* Read the chunk info file if not a compressed file */
 
 	if (!compressed_file)
 	{
+	    if (lseek(private_file_desc, FILE_BLOCK_SIZE, SEEK_SET) < 0)
+	    {
+		perror("chunk index");
+		exit(1);
+	    }
+
+	    chunk_info_size_in_bytes = nwos_total_private_chunks * sizeof(Chunk_Info);
+	    chunk_info = nwos_malloc(chunk_info_size_in_bytes);
+
+	    if (read(private_file_desc, chunk_info, chunk_info_size_in_bytes) != chunk_info_size_in_bytes)
+	    {
+		snprintf(log_msg, sizeof(log_msg), "reading chunk info from: %s", nwos_private_path);
+		perror(log_msg);
+		exit(1);
+	    }
+
+	    used_blocks = 0;
+	    for (i = 0; i < (int)nwos_total_private_chunks; i++)
+	    {
+#ifndef WORDS_BIGENDIAN
+		chunk_info[i].ref = byteswap_uint32(chunk_info[i].ref);
+		chunk_info[i].used = byteswap_uint16(chunk_info[i].used);
+		chunk_info[i].index = byteswap_uint16(chunk_info[i].index);
+#endif
+		used_blocks += chunk_info[i].used;
+
+		if (chunk_info[i].ref > MAXIMUM_PRIVATE_REFERENCE)
+		{
+		    snprintf(log_msg, sizeof(log_msg), 
+			     "WARNING: you have a chunk of data located at %08x that is above %08x",
+			     chunk_info[i].ref, MAXIMUM_PRIVATE_REFERENCE);
+		    nwos_log(log_msg);
+		    fprintf(stderr, "%s,\n", log_msg);
+		    fprintf(stderr, "which is used for temporary storage in versions after Alpha_29.4.  Please\n"
+			    "e-mail me at qrwsoftware@gmail.com for help fixing this.\n\n");
+		    sleep(4);
+		}
+	    }
+
+	    if (used_blocks != nwos_used_private_blocks)
+	    {
+		snprintf(log_msg, sizeof(log_msg), 
+			 "Warning: calculated sum of used blocks (%u) doesn't match stored (%u)",
+			 used_blocks, nwos_used_private_blocks);
+		nwos_log(log_msg);
+		fprintf(stderr, "%s\n", log_msg);
+	    }
+
+	    chunk_info_modified = false;
+
+	    update_chunk_info_reverse_index();
+
+	    /* now create the chunk_used_index */
+
+	    chunk_info_size_in_bytes = nwos_total_private_chunks * sizeof(uint16);
+	    chunk_used_index = nwos_malloc(chunk_info_size_in_bytes);
+
+	    /* insert them into a heap to do a heap sort */
+	    for (i = 0; i < (int)nwos_used_private_chunks; i++)
+	    {
+		j = i + 1;
+		while (j / 2 > 0 && chunk_info[chunk_used_index[(j / 2) - 1]].used < chunk_info[i].used)
+		{
+		    /* parent is smaller, move it lower in the tree */
+		    chunk_used_index[j - 1] = chunk_used_index[(j / 2) - 1];
+		    j = j / 2;
+		}
+
+		chunk_used_index[j - 1] = i;
+	    }
+
+	    for (i = (int)nwos_used_private_chunks; i > 1; i--)
+	    {
+		/* save the index at the top of the tree (smallest) */
+		save = chunk_used_index[i - 1];
+
+		/* move the root of the the (largest) to the top */
+		chunk_used_index[i - 1] = chunk_used_index[0];
+
+		j = 1;
+		while (j < (i / 2) - 1)
+		{
+		    k = j * 2;
+
+		    /* if there is a right child */
+		    if (k < i - 1)
+		    {
+			left = chunk_info[chunk_used_index[k - 1]].used;
+			right = chunk_info[chunk_used_index[k + 1 - 1]].used;
+
+			if (left < right)
+			{
+			    k++;
+			}
+		    }
+
+		    if (chunk_info[save].used >= chunk_info[chunk_used_index[k-1]].used) break;
+
+		    chunk_used_index[j - 1] = chunk_used_index[k - 1];
+
+		    j = k;
+		}
+
+		chunk_used_index[j - 1] = save;
+	    }
+
 	    assert(nwos_total_private_blocks > 0);
 	}
 
@@ -1012,7 +1133,7 @@ void nwos_initialize_disk_io(AccessType type, const char* path)
 
     backup_dir = nwos_get_backup_directory_path();
 
-    if (backup_dir != NULL && !read_only)
+    if (backup_dir != NULL && *backup_dir != '\0' && !read_only)
     {
 	if (!nwos_initialize_backup(backup_dir, &nwos_disk_header))    /* creating backup failed, shut 'er back down */
 	{
@@ -1176,7 +1297,7 @@ void nwos_allocate_new_chunk(uint32 ref)
 
     assert(nwos_used_private_chunks < nwos_total_private_chunks);
 
-    assert(RESERVED_PUBLIC_BLOCKS <= ref && ref <= MAXIMUM_VALID_PRIVATE_REFERENCE);
+    assert(MINIMUM_PRIVATE_REFERENCE <= ref && ref <= MAXIMUM_PRIVATE_REFERENCE);
 
     assert(nwos_hash_uint32_ref(ref) == 0);
 
@@ -1191,7 +1312,7 @@ void nwos_allocate_new_chunk(uint32 ref)
     }
     i++;
 
-    offset = (ref - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
+    offset = (ref - MINIMUM_PRIVATE_REFERENCE) % USABLE_BLOCKS_PER_CHUNK;
 
     chunk_info[i].ref = ref - offset;
 
@@ -1200,7 +1321,7 @@ void nwos_allocate_new_chunk(uint32 ref)
 
     /* make sure chunk_info.ref is an even multiple of USABLE_BLOCKS_PER_CHUNK */
 
-    offset = chunk_info[i].ref - RESERVED_PUBLIC_BLOCKS;
+    offset = chunk_info[i].ref - MINIMUM_PRIVATE_REFERENCE;
 
     chunk_info[i].used = 0;
     chunk_info[i].index = nwos_used_private_chunks;
@@ -1278,7 +1399,7 @@ static int uint32_ref_to_info_index(uint32 ref)
     int mid = 0;
 
     assert(nwos_used_private_chunks > 0);
-    assert(RESERVED_PUBLIC_BLOCKS <= ref && ref <= MAXIMUM_VALID_PRIVATE_REFERENCE);
+    assert(MINIMUM_PRIVATE_REFERENCE <= ref && ref <= MAXIMUM_PRIVATE_REFERENCE);
 
     lwr = 1;
 
@@ -1334,11 +1455,11 @@ uint32 nwos_hash_uint32_ref(uint32 ref)
     uint32 chunk;
     uint32 offset;
 
-    if (ref < RESERVED_PUBLIC_BLOCKS)
+    if (ref <= MAXIMUM_PUBLIC_REFERENCE)
     {
 	result = ref;
     }
-    else if (ref <= MAXIMUM_VALID_PRIVATE_REFERENCE && nwos_used_private_chunks > 0)
+    else if (ref <= MAXIMUM_PRIVATE_REFERENCE && nwos_used_private_chunks > 0)
     {
 	index = uint32_ref_to_info_index(ref);
 
@@ -1346,7 +1467,7 @@ uint32 nwos_hash_uint32_ref(uint32 ref)
 	{
 	    chunk = (uint32) chunk_info[index].index;
 
-	    offset = (ref - RESERVED_PUBLIC_BLOCKS) % USABLE_BLOCKS_PER_CHUNK;
+	    offset = (ref - MINIMUM_PRIVATE_REFERENCE) % USABLE_BLOCKS_PER_CHUNK;
 
 	    result = nwos_block_offset_to_chunks + (chunk * BLOCKS_IN_CHUNK) + BIT_MAP_BLOCKS + offset;
 	}
@@ -1383,6 +1504,19 @@ static inline off_t ref_to_offset(ObjRef* ref)
 }
 
 
+/**********************************************************************************************/
+/* This function returns the index into the chunk_info table for the chunk index passed in or */
+/* -1 if not found.                                                                           */
+/**********************************************************************************************/
+
+static int chunk_index_to_info_index(uint16 chunk_index)
+{
+    assert(chunk_index < nwos_used_private_chunks);
+
+    return chunk_info_reverse_index[chunk_index];
+}
+
+
 /*********************************************/
 /* Functions to manipulate the bit map cache */
 /*********************************************/
@@ -1411,16 +1545,22 @@ static int8 bits_in_byte[256] =
 void write_bit_map(Bit_Map_Cache_Entry* entry)
 {
     int i;
+    int info_index;
     uint32 chunk;
     uint32 blocks_used = 0;
     char log_msg[80];
 
-    assert(0 <= entry->index && entry->index < nwos_used_private_chunks);
+    assert(0 <= entry->chunk_index && entry->chunk_index < nwos_used_private_chunks);
 
-    chunk = nwos_block_offset_to_chunks + chunk_info[entry->index].index * BLOCKS_IN_CHUNK;
+    info_index = chunk_index_to_info_index(entry->chunk_index);
+
+    assert(0 <= info_index && info_index < nwos_used_private_chunks);
+    assert(chunk_info[info_index].index == entry->chunk_index);
+
+    chunk = nwos_block_offset_to_chunks + entry->chunk_index * BLOCKS_IN_CHUNK;
 
 #ifdef LOG_BIT_MAP
-    snprintf(log_msg, sizeof(log_msg), "write bit_map: %d - %08x", entry->index, chunk);
+    snprintf(log_msg, sizeof(log_msg), "write bit_map - info_index: %d  chunk: %d - %08x", info_index, entry->chunk_index, chunk);
     nwos_log(log_msg);
 #endif
 
@@ -1434,15 +1574,15 @@ void write_bit_map(Bit_Map_Cache_Entry* entry)
 
     assert(blocks_used <= BLOCKS_IN_CHUNK - BIT_MAP_BLOCKS);
 
-    if (blocks_used != chunk_info[entry->index].used)
+    if (blocks_used != chunk_info[info_index].used)
     {
 	snprintf(log_msg, sizeof(log_msg),
 		 "Warning: blocks used in chunk mismatch - map: %u  stored: %u",
-		 blocks_used, chunk_info[entry->index].used);
+		 blocks_used, chunk_info[info_index].used);
 	perror(log_msg);
 	fprintf(stderr, "%s\n", log_msg);
 
-	chunk_info[entry->index].used = blocks_used;
+	chunk_info[info_index].used = blocks_used;
 	chunk_info_modified = true;
     }
 
@@ -1469,6 +1609,7 @@ void write_bit_map(Bit_Map_Cache_Entry* entry)
 void read_bit_map_into_cache(Bit_Map_Cache_Entry* entry)
 {
     int i;
+    int info_index;
     uint32 chunk;
     uint32 blocks_used = 0;
     char log_msg[80];
@@ -1478,7 +1619,14 @@ void read_bit_map_into_cache(Bit_Map_Cache_Entry* entry)
     nwos_log(log_msg);
 #endif
 
-    chunk = nwos_block_offset_to_chunks + chunk_info[entry->index].index * BLOCKS_IN_CHUNK;
+    assert(0 <= entry->chunk_index && entry->chunk_index < nwos_used_private_chunks);
+
+    info_index = chunk_index_to_info_index(entry->chunk_index);
+
+    assert(0 <= info_index && info_index < nwos_used_private_chunks);
+    assert(chunk_info[info_index].index == entry->chunk_index);
+
+    chunk = nwos_block_offset_to_chunks + entry->chunk_index * BLOCKS_IN_CHUNK;
 
     assert(chunk < nwos_block_offset_to_chunks + nwos_total_private_blocks);
 
@@ -1511,15 +1659,15 @@ void read_bit_map_into_cache(Bit_Map_Cache_Entry* entry)
     assert(blocks_used <= BLOCKS_IN_CHUNK - BIT_MAP_BLOCKS);
 
 
-    if (blocks_used != chunk_info[entry->index].used)
+    if (blocks_used != chunk_info[info_index].used)
     {
 	snprintf(log_msg, sizeof(log_msg),
 		 "Warning: blocks used in chunk mismatch - map: %u  stored: %u",
-		 blocks_used, chunk_info[entry->index].used);
+		 blocks_used, chunk_info[info_index].used);
 	perror(log_msg);
 	fprintf(stderr, "%s\n", log_msg);
 
-	chunk_info[entry->index].used = blocks_used;
+	chunk_info[info_index].used = blocks_used;
 	chunk_info_modified = true;
     }
 
@@ -1528,23 +1676,9 @@ void read_bit_map_into_cache(Bit_Map_Cache_Entry* entry)
 }
 
 
-/**********************************************************************************************/
-/* This function returns the index into the chunk_info table for the chunk index passed in or */
-/* -1 if not found.                                                                           */
-/**********************************************************************************************/
-
-static int chunk_index_to_info_index(uint16 chunk_index)
-{
-    assert(chunk_index < nwos_used_private_chunks);
-
-    return chunk_info_reverse_index[chunk_index];
-}
-
-
 static Bit_Map_Cache_Entry* find_bit_map_in_cache(uint32 block)
 {
     int i;
-    int info_index;
     uint16 chunk_index;
     Bit_Map_Cache_Entry* result = NULL;
 
@@ -1554,14 +1688,11 @@ static Bit_Map_Cache_Entry* find_bit_map_in_cache(uint32 block)
 
     chunk_index = (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK;
 
-    info_index = chunk_index_to_info_index(chunk_index);
-
-    assert(info_index >= 0);   /* should never call this if the chunk this block is in isn't used */
 
     /* see if it's already in the cache */
     for (i = 0; i < BIT_MAP_CACHE_SIZE; i++)
     {
-	if (bit_map_cache[i].in_use && bit_map_cache[i].index == info_index) break;
+	if (bit_map_cache[i].in_use && bit_map_cache[i].chunk_index == chunk_index) break;
     }
 
     if (i < BIT_MAP_CACHE_SIZE)   /* found it */
@@ -1593,7 +1724,7 @@ static Bit_Map_Cache_Entry* find_bit_map_in_cache(uint32 block)
 	    write_bit_map(result);
 	}
 
-	result->index = info_index;
+	result->chunk_index = chunk_index;
 
 	read_bit_map_into_cache(result);
     }
@@ -1611,6 +1742,7 @@ void nwos_set_bit_in_map(uint32 block)
 {
     int byte_num;
     int bit_num;
+    int info_index;
     Bit_Map_Cache_Entry* entry;
 
     entry = find_bit_map_in_cache(block);
@@ -1627,7 +1759,12 @@ void nwos_set_bit_in_map(uint32 block)
 
 	entry->dirty = true;
 
-	chunk_info[entry->index].used++;
+	info_index = chunk_index_to_info_index(entry->chunk_index);
+
+	assert(0 <= info_index && info_index < nwos_used_private_chunks);
+	assert(chunk_info[info_index].index == entry->chunk_index);
+
+	chunk_info[info_index].used++;
 	chunk_info_modified = true;
 
 	nwos_used_private_blocks++;
@@ -1639,6 +1776,7 @@ void nwos_clear_bit_in_map(uint32 block)
 {
     int byte_num;
     int bit_num;
+    int info_index;
     Bit_Map_Cache_Entry* entry;
 
     entry = find_bit_map_in_cache(block);
@@ -1655,7 +1793,12 @@ void nwos_clear_bit_in_map(uint32 block)
 
 	entry->dirty = true;
 
-	chunk_info[entry->index].used--;
+	info_index = chunk_index_to_info_index(entry->chunk_index);
+
+	assert(0 <= info_index && info_index < nwos_used_private_chunks);
+	assert(chunk_info[info_index].index == entry->chunk_index);
+
+	chunk_info[info_index].used--;
 	chunk_info_modified = true;
 
 	nwos_used_private_blocks--;
@@ -1952,10 +2095,10 @@ void nwos_restart_id_generation()
 }
 
 
-void nwos_set_allocation_parameters(uint32 density, int32 skip_backward, int32 skip_forward)
+void nwos_set_allocation_parameters(uint32 density, uint32 skip_backward, uint32 skip_forward)
 {
     assert(1 <= density && density <= USABLE_BLOCKS_PER_CHUNK);
-    assert(-(int)RESERVED_PRIVATE_CHUNKS <= skip_backward && skip_backward <= 0);
+    assert(skip_backward <= RESERVED_PRIVATE_CHUNKS);
     assert(1 <= skip_forward && skip_forward <= RESERVED_PRIVATE_CHUNKS);
 
     chunk_density_target = density;
@@ -1973,11 +2116,12 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
     uint32 ref;
     char msg[80];
     bool need_to_allocate_more_chunks;
+    int i;
 
     assert(estimated_blocks > 0);
 
     assert(1 <= chunk_density_target && chunk_density_target <= USABLE_BLOCKS_PER_CHUNK);
-    assert(-(int)RESERVED_PRIVATE_CHUNKS <= chunk_skip_backward && chunk_skip_backward <= 0);
+    assert(chunk_skip_backward <= RESERVED_PRIVATE_CHUNKS);
     assert(1 <= chunk_skip_forward && chunk_skip_forward <= RESERVED_PRIVATE_CHUNKS);
 
     if (block_estimate > 0)   /* there was a previous estimate */
@@ -2025,34 +2169,45 @@ bool nwos_check_blocks_available(uint32 estimated_blocks)
 	need_to_allocate_more_chunks = allocated_blocks <= nwos_used_private_blocks + block_estimate;
     }
 
+    assert(nwos_used_private_blocks + block_estimate < total_blocks);
+
     while (need_to_allocate_more_chunks)
     {
-	/* allocate can currently only deal with adding chunks at the end */
-	if (chunk_info[nwos_used_private_chunks-1].index + 1 >= nwos_total_private_chunks)
-	{
-	    fprintf(stderr, "This version cannot allocate chunks below what has already been allocated,\n");
-	    fprintf(stderr, "since no more blocks are available at the end, cannot import this file.\n");
+	assert(nwos_used_private_chunks < nwos_total_private_chunks);
 
-	    return false;
+	/* temporary hack to handle the situation where we have reached the end of the private chunks */
+	if (chunk_info[nwos_used_private_chunks-1].ref + USABLE_BLOCKS_PER_CHUNK > MAXIMUM_PRIVATE_REFERENCE)
+	{
+	    snprintf(msg, sizeof(msg), "WARNING: reached the end of private space, working backwards.");
+	    nwos_log(msg);
+	    fprintf(stderr, "%s\n", msg);
+
+	    for (i = nwos_used_private_chunks - 1; i > 0; i--)
+	    {
+		if (chunk_info[i-1].ref < chunk_info[i].ref - USABLE_BLOCKS_PER_CHUNK) break;
+	    }
+
+	    ref = chunk_info[i].ref - USABLE_BLOCKS_PER_CHUNK;
 	}
-
-	if (chunk_skip_forward == 1)
+	else if (chunk_skip_forward == 1)
 	{
-	    skip = 1;
+	    ref = chunk_info[nwos_used_private_chunks-1].ref + USABLE_BLOCKS_PER_CHUNK;
 	}
 	else if (chunk_skip_forward == RESERVED_PRIVATE_CHUNKS)
 	{
-	    max = MAXIMUM_VALID_PRIVATE_REFERENCE - chunk_info[nwos_used_private_chunks-1].ref;
+	    max = MAXIMUM_PRIVATE_REFERENCE - chunk_info[nwos_used_private_chunks-1].ref;
 	    skip = 1 + random() % (max / USABLE_BLOCKS_PER_CHUNK);
+	    ref = chunk_info[nwos_used_private_chunks-1].ref + (USABLE_BLOCKS_PER_CHUNK * skip);
 	}
 	else
 	{
-	    skip = 1 + random() % chunk_skip_forward;
+	    max = (MAXIMUM_PRIVATE_REFERENCE - chunk_info[nwos_used_private_chunks-1].ref) / USABLE_BLOCKS_PER_CHUNK;
+	    if (max > chunk_skip_forward) max = chunk_skip_forward;
+	    skip = 1 + random() % max;
+	    ref = chunk_info[nwos_used_private_chunks-1].ref + (USABLE_BLOCKS_PER_CHUNK * skip);
 	}
 
-	ref = chunk_info[nwos_used_private_chunks-1].ref + (USABLE_BLOCKS_PER_CHUNK * skip);
-
-	snprintf(msg, sizeof(msg), "Allocating chunk: %08x\n", ref);
+	snprintf(msg, sizeof(msg), "Allocating chunk: %08x", ref);
 	nwos_log(msg);
 
 	nwos_allocate_new_chunk(ref);
@@ -2153,12 +2308,16 @@ void nwos_generate_new_closely_spaced_id(int density, ObjRef* new_ref)
 	    block = nwos_block_offset_to_chunks + chunk_info[info_index].index * BLOCKS_IN_CHUNK + BIT_MAP_BLOCKS + (byte_num * 8);
 	}
 
-	if (entry == NULL || chunk_info[entry->index].index != (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK)
+	if (entry == NULL || info_index < 0 || entry->chunk_index != (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK)
 	{
 	    entry = find_bit_map_in_cache(block);
+
+	    info_index = chunk_index_to_info_index(entry->chunk_index);
 	}
 
-	assert(chunk_info[entry->index].index == (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK);
+	assert(0 <= info_index && info_index < nwos_used_private_chunks);
+	assert(chunk_info[info_index].index == entry->chunk_index);
+	assert(chunk_info[info_index].index == (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK);
 
 	byte_num = (block % BLOCKS_IN_CHUNK) / 8;
 
@@ -2214,9 +2373,9 @@ void nwos_generate_new_closely_spaced_id(int density, ObjRef* new_ref)
 
     nwos_set_bit_in_map(block);
 
-    assert(chunk_info[entry->index].index == (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK);
+    assert(entry->chunk_index == (block - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK);
 
-    ref = chunk_info[entry->index].ref + (block - nwos_block_offset_to_chunks - chunk_info[entry->index].index * BLOCKS_IN_CHUNK - BIT_MAP_BLOCKS);
+    ref = chunk_info[info_index].ref + (block - nwos_block_offset_to_chunks - chunk_info[info_index].index * BLOCKS_IN_CHUNK - BIT_MAP_BLOCKS);
 
     nwos_word_to_ref(ref, new_ref);
 
@@ -2236,6 +2395,7 @@ void nwos_generate_new_1_in_N_id(uint32 n, ObjRef* new_ref)
     Bit_Map_Cache_Entry* entry = NULL;
     uint32 ref;
     uint32 hash;
+    int    info_index = -1;
     int    byte_num;
     int    bit_num;
     uint8  byte;
@@ -2268,12 +2428,16 @@ void nwos_generate_new_1_in_N_id(uint32 n, ObjRef* new_ref)
 	    hash = nwos_hash_uint32_ref(ref);
 	}	    
 
-	if (entry == NULL || chunk_info[entry->index].index != (hash - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK)
+	if (entry == NULL || info_index < 0 || entry->chunk_index != (hash - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK)
 	{
 	    entry = find_bit_map_in_cache(hash);
+
+	    info_index = chunk_index_to_info_index(entry->chunk_index);
 	}
 
-	assert(chunk_info[entry->index].index == (hash - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK);
+	assert(0 <= info_index && info_index < nwos_used_private_chunks);
+	assert(chunk_info[info_index].index == entry->chunk_index);
+	assert(chunk_info[info_index].index == (hash - nwos_block_offset_to_chunks) / BLOCKS_IN_CHUNK);
 
 	byte_num = (hash % BLOCKS_IN_CHUNK) / 8;
 	assert(byte_num < BIT_MAP_BYTES);
@@ -2371,7 +2535,7 @@ bool nwos_read_block(ObjRef* ref, uint8 block[FILE_BLOCK_SIZE])
     return cache[i] != NULL;
 
 #else
-    if (nwos_ref_to_word(ref) < RESERVED_PUBLIC_BLOCKS)
+    if (nwos_ref_to_word(ref) <= MAXIMUM_PUBLIC_REFERENCE)
     {
 	file_desc = public_file_desc;
 	offset = (off_t) nwos_ref_to_word(ref) * FILE_BLOCK_SIZE;
