@@ -27,6 +27,17 @@
 --
 --
 -- $Log: disk_usage.c,v $
+-- Revision 1.27  2008/10/20 02:20:24  jsedwards
+-- Fix Bug #2117258 - Seg fault running disk_usage.  Changed for loop to only
+-- do used_private_chunks instead of total_private_chunks because only space
+-- for used_private_chunks is malloc'd.
+--
+-- Revision 1.26  2008/09/17 10:58:25  jsedwards
+-- Added display_graph function to print bar graph of usage.
+--
+-- Revision 1.25  2008/09/16 13:20:37  jsedwards
+-- Change to print a progress bar when not verbose instead of blocks used.
+--
 -- Revision 1.24  2008/08/10 15:23:25  jsedwards
 -- Added path to "Missing magic number" and "Incorrect version in header" error
 -- messages.
@@ -192,13 +203,17 @@ uint32 calculate_blocks_used(int obj_fd, uint32 start, Chunk_Info chunk_info[],
     off_t block;
     size_t bytes_read;
     uint8 map[BIT_MAP_BYTES];
-    uint32 last_total = 0;
     uint32 blocks_used;
     uint32 result = 0;
     int i;
     int j;
 
 
+    if (!verbose)
+    {
+	nwos_start_progress_bar();
+    }
+
     for (i = 0; i < num_chunks; i++)
     {
 	block = start + chunk_info[i].index * BLOCKS_IN_CHUNK;
@@ -232,25 +247,115 @@ uint32 calculate_blocks_used(int obj_fd, uint32 start, Chunk_Info chunk_info[],
 	    printf("Chunk %d  ref: %08x  index: %u  used: %u\n",
 		   i, chunk_info[i].ref, chunk_info[i].index, chunk_info[i].used);
 	}
-	else if ((block - start) % 4194304 == 0)
+	else
 	{
-	    printf("%u - %u  used: %d\n",
-		   (uint32)((block - start) / 4194304), (uint32)(block - start), blocks_used - last_total);
-	    fflush(stdout);
-	    last_total = result;
+	    nwos_update_progress_bar((float)i / (float)num_chunks);
 	}
 
 	if ((blocks_used - BIT_MAP_BLOCKS) != chunk_info[i].used)
 	{
+	    if (!verbose)
+	    {
+		nwos_erase_progress_bar();
+	    }
+
 	    fprintf(stderr, "Error: chunk %d index %u stored count (%u) doesn't match bit map count (%u)\n",
 		    i, chunk_info[i].index, chunk_info[i].used, blocks_used - BIT_MAP_BLOCKS);
+
+	    if (!verbose)
+	    {
+		nwos_start_progress_bar();
+	    }
 	}
     }
 
+    if (!verbose)
+    {
+	nwos_finish_progress_bar();
+    }
+
     return result;
 }
 
 
+void display_graph(Chunk_Info chunk_info[], uint32 num_chunks)
+{
+    const int graph_bits = 6;
+    const int divisor = 1 << (graph_bits - 4);
+    uint32 used_blocks[1 << graph_bits];
+    uint32 total_blocks[1 << graph_bits];
+    int segment;
+    int percent;
+    
+    int i;
+
+    memset(used_blocks, 0, sizeof(used_blocks));
+    memset(total_blocks, 0, sizeof(total_blocks));
+
+    for (i = 0; i < num_chunks; i++)
+    {
+	segment = chunk_info[i].ref >> (32 - graph_bits);
+
+	assert((RESERVED_PUBLIC_BLOCKS >> (32 - graph_bits)) <= segment && segment < (1 << graph_bits));
+
+	total_blocks[segment] += USABLE_BLOCKS_PER_CHUNK;
+
+	used_blocks[segment] += chunk_info[i].used;
+    }
+
+    for (percent = 100; percent >= 0; percent -= 5)
+    {
+	if (percent % 10 == 0)
+	{
+	    printf("%3d |", percent);
+	}
+	else
+	{
+	    printf("    |");
+	}
+
+	for (i = (RESERVED_PUBLIC_BLOCKS >> (32 - graph_bits)); i < (1 << graph_bits); i++)
+	{
+	    if (total_blocks[i] > 0 && ((used_blocks[i] * 100) / total_blocks[i]) >= (uint32)percent)
+	    {
+		putchar('*');
+	    }
+	    else
+	    {
+		putchar(' ');
+	    }
+	}
+
+	putchar('\n');
+    }
+
+    printf("    +");
+    for (i = (RESERVED_PUBLIC_BLOCKS >> (32 - graph_bits)); i < (1 << graph_bits); i++) putchar('-');
+    putchar('\n');
+
+    printf("     ");
+    for (i = (RESERVED_PUBLIC_BLOCKS >> (32 - graph_bits)); i < (1 << graph_bits); i++)
+    {
+	if (i % divisor == 0)
+	{
+	    if (i / divisor < 10)
+	    {
+		putchar('0' + i / divisor);
+	    }
+	    else
+	    {
+		putchar('A' + i / divisor - 10);
+	    }
+	}
+	else
+	{
+	    putchar(' ');
+	}
+    }
+    putchar('\n');
+}
+
+
 int main(int argc, char* argv[])
 {
     int obj_file_desc;
@@ -371,7 +476,7 @@ int main(int argc, char* argv[])
     total_private_chunks = reserved_private_blocks / BLOCKS_IN_CHUNK;
 
     used_blocks = 0;
-    for (i = 0; i < (int)total_private_chunks; i++)
+    for (i = 0; i < (int)used_private_chunks; i++)
     {
 #ifndef WORDS_BIGENDIAN
 	chunk_info[i].ref = byteswap_uint32(chunk_info[i].ref);
@@ -455,6 +560,12 @@ int main(int argc, char* argv[])
 
     printf("\n");
 
+
+    display_graph(chunk_info, used_private_chunks);
+
+
+    printf("\n");
+
     fflush(stdout);
 
     return 0;
