@@ -27,6 +27,18 @@
 --
 --
 -- $Log: import_file.c,v $
+-- Revision 1.40  2008/09/01 18:53:48  jsedwards
+-- Fixed Bug #2043130 - asserts if given an absolute path.
+--
+-- Revision 1.39  2008/09/01 03:13:42  jsedwards
+-- Change for new nwos_initialize_objectify calling convention (doesn't pass
+-- back root_object_reference anymore) and removed call to nwos_set_root_object
+-- because initialize calls it now.
+--
+-- Revision 1.38  2008/08/30 12:46:12  jsedwards
+-- Removed code and variables to read pass phrase and pass it to initialize,
+-- and change parameters passed to initialize.
+--
 -- Revision 1.37  2008/07/13 14:18:58  jsedwards
 -- Feature Request #1856369 - Added handling of new IMPORTED_DATA result from
 -- nwos_create_file.
@@ -412,11 +424,6 @@ void process_file(const char* path, Add_Revision_Type add_revision, bool do_recu
 int main(int argc, char* argv[])
 {
     struct stat stat_struct;
-    ObjRef root_object_ref;
-    uint8 big_key[16 + 8 + 4];
-    uint8 bf_key[16];
-    uint32 linear;
-    uint32 serial;
     int i;
     int j;
     char path[PATH_MAX];
@@ -509,16 +516,7 @@ int main(int argc, char* argv[])
 
     nwos_log_arguments(argc, argv);
 
-    nwos_get_key_from_password(big_key, sizeof(big_key));
-
-    memcpy(bf_key, big_key, 16);
-    linear = ((uint32)big_key[16] << 24) | ((uint32)big_key[17] << 16) | ((uint32)big_key[18] << 8) | (uint32)big_key[19];
-    memcpy(root_object_ref.id, big_key+20, 4);
-    serial = ((uint32)big_key[24] << 24) | ((uint32)big_key[25] << 16) | ((uint32)big_key[26] << 8) | (uint32)big_key[27];
-
-    nwos_initialize_objectify(bf_key, linear, serial, READ_WRITE, DEFAULT_FILE);
-
-    nwos_set_root_object(&root_object_ref);
+    nwos_initialize_objectify(READ_WRITE, DEFAULT_FILE);
 
     nwos_set_security_level(Security_Low);
 
@@ -537,7 +535,11 @@ int main(int argc, char* argv[])
 
 		if (strlen(argv[i]) + 1 > sizeof(path))
 		{
-		    fprintf(stderr, "ERROR: the path '%s' is too long for the buffer (exceeds %d characters), skipped\n", argv[i], sizeof(path));
+		    fprintf(stderr, "WARNING: the path '%s' is too long for the buffer (exceeds %d characters) - ignored.\n", argv[i], sizeof(path));
+		}
+		else if (*argv[i] == '/')
+		{
+		    fprintf(stderr, "WARNING: the path '%s' is absolute and this version cannot process it - ignored.\n", argv[i]);
 		}
 		else
 		{
@@ -577,7 +579,11 @@ int main(int argc, char* argv[])
 	{
 	    if (strlen(argv[i]) + 1 > sizeof(path))
 	    {
-		fprintf(stderr, "ERROR: the path '%s' is too long for the buffer (exceeds %d characters), skipped\n", argv[i], sizeof(path));
+		fprintf(stderr, "WARNING: the path '%s' is too long for the buffer (exceeds %d characters) - ignored.\n", argv[i], sizeof(path));
+	    }
+	    else if (*argv[i] == '/')
+	    {
+		fprintf(stderr, "WARNING: the path '%s' is absolute and this version cannot process it - ignored.\n", argv[i]);
 	    }
 	    else
 	    {
