@@ -27,6 +27,34 @@
 --
 --
 -- $Log: objectify_private.h,v $
+-- Revision 1.78  2008/10/31 12:35:31  jsedwards
+-- Added MAXIMUM_PUBLIC_REFERENCE, MINIMUM_PRIVATE_REFERENCE,
+-- MINIMUM_TEMPORARY_REFERENCE, MAXIMUM_TEMPORARY_REFERENCE, and changed
+-- MAXIMUM_VALID_PRIVATE_REFERENCE to MAXIMUM_PRIVATE_REFERENCE.
+--
+-- Revision 1.77  2008/10/28 10:14:58  jsedwards
+-- Changed skip_forward and skip_backward parameters to unsigned.
+--
+-- Revision 1.76  2008/10/20 13:41:19  jsedwards
+-- Changed max root transformations back to 128, now that it is an add instead
+-- of multiplying by prime numbers.
+--
+-- Revision 1.75  2008/10/03 11:52:02  jsedwards
+-- Changed MAX_TRANSFORMATIONS to MAX_ROOT_TRANSFORMATIONS and reduced it from
+-- 128 to 16 because using prime numbers now.
+--
+-- Revision 1.74  2008/10/03 04:15:30  jsedwards
+-- Added MAX_TRANSFORMATIONS and external declaration for nwos_transform_root.
+--
+-- Revision 1.73  2008/09/19 13:09:52  jsedwards
+-- Remove #if that was testing for version string = 0029 because the compiler
+-- was not happy about it.
+--
+-- Revision 1.72  2008/09/18 14:14:04  jsedwards
+-- Changed to reserve 256 megabytes for temporary space (one mega blocks).  This
+-- is kludged for Alpha_29 because the chunks don't come out even, instead of
+-- the maximum valid reference being 0xffefffff it is 0xffeffe40.
+--
 -- Revision 1.71  2008/09/01 18:16:29  jsedwards
 -- Changed nwos_create_root to not take any parameters.
 --
@@ -321,7 +349,9 @@
 
 #include "objectify.h"
 
-#define RESERVED_PUBLIC_BLOCKS 0x10000000    /* IDs 0x00000000 to 0x0fffffff reserved */
+#define RESERVED_PUBLIC_BLOCKS    0x10000000    /* IDs 0x00000000 to 0x0fffffff reserved */
+#define RESERVED_PRIVATE_BLOCKS   0xeff00000    /* IDs 0x10000000 to 0xfff00000 reserved */
+#define RESERVED_TEMPORARY_BLOCKS 0x00100000    /* IDs 0xfff00000 to 0xffffffff reserved */
 
 #define BIT_MAP_BLOCKS 32
 #define BIT_MAP_BYTES (BIT_MAP_BLOCKS * FILE_BLOCK_SIZE)
@@ -333,15 +363,22 @@
 
 /* some blocks don't come out even when dividing usable chunks per block, so don't allocate them */
 
-#define RESERVED_PRIVATE_CHUNKS ((0xffffffff - RESERVED_PUBLIC_BLOCKS) / USABLE_BLOCKS_PER_CHUNK)
-/*                              ((0xffffffff - 0x10000000)             / 65504)    = 61470 */
+#define RESERVED_PRIVATE_CHUNKS (RESERVED_PRIVATE_BLOCKS / USABLE_BLOCKS_PER_CHUNK)
+/*                              (0xeff00000              / 65504)    = 61454 */
 
-#define RESERVED_PRIVATE_BLOCKS (RESERVED_PRIVATE_CHUNKS * USABLE_BLOCKS_PER_CHUNK)
-/*                                                 61470 * 65504                   = 4026530880 */
-/*                                                                                 = 0xeffffc40 */
+#define MAXIMUM_PUBLIC_REFERENCE   (RESERVED_PUBLIC_BLOCKS - 1)
 
-#define MAXIMUM_VALID_PRIVATE_REFERENCE (RESERVED_PUBLIC_BLOCKS + RESERVED_PRIVATE_BLOCKS - 1)
-/*                                                   0x10000000 + 0xeffffc40 - 1   = 0xfffffc3f */
+#define MINIMUM_PRIVATE_REFERENCE  RESERVED_PUBLIC_BLOCKS
+
+// USE THIS in Alpha_30:
+// #define MAXIMUM_PRIVATE_REFERENCE (RESERVED_PUBLIC_BLOCKS + RESERVED_PRIVATE_BLOCKS - 1)
+/*                                                0x10000000 + 0xeff00000 - 1   = 0xffefffff */
+
+// THIS IS ONLY FOR Alpha_29
+#define MAXIMUM_PRIVATE_REFERENCE (RESERVED_PUBLIC_BLOCKS + (RESERVED_PRIVATE_CHUNKS * USABLE_BLOCKS_PER_CHUNK) - 1) /* 0xffeffe40 */
+
+#define MINIMUM_TEMPORARY_REFERENCE (RESERVED_PUBLIC_BLOCKS + RESERVED_PRIVATE_BLOCKS)
+#define MAXIMUM_TEMPORARY_REFERENCE (RESERVED_PUBLIC_BLOCKS + RESERVED_PRIVATE_BLOCKS + (RESERVED_TEMPORARY_BLOCKS - 1))
 
 #define CHUNK_SIZE (BLOCKS_IN_CHUNK * FILE_BLOCK_SIZE)
 
@@ -446,6 +483,10 @@ typedef struct {
 
 extern bool verbose;
 
+#define MAX_ROOT_TRANSFORMATIONS 128
+
+extern void nwos_transform_root(ObjRef* dst_ref, ObjRef* src_ref, int num);
+
 extern uint8 nwos_random_sequence[NUM_STORED_SEQ][FILE_BLOCK_SIZE];     /* resequencing tables */
 
 extern ObjRef nwos_public_class_definition_class_ref;
@@ -507,7 +548,7 @@ extern void  nwos_terminate_disk_io();
 extern bool  nwos_initialize_backup(const char* path, Disk_Header* disk_header);
 extern void  nwos_terminate_backup(bool modified, Disk_Header* disk_header);
 extern void  nwos_backup_write_block(uint8 block[FILE_BLOCK_SIZE]);
-extern void  nwos_set_allocation_parameters(uint32 density, int32 skip_backward, int32 skip_forward);
+extern void  nwos_set_allocation_parameters(uint32 density, uint32 skip_backward, uint32 skip_forward);
 extern void  nwos_allocate_new_chunk(uint32 ref);
 extern void nwos_generate_new_closely_spaced_id(int density, ObjRef* new_ref);
 extern void nwos_generate_new_1_in_N_id(uint32 n, ObjRef* new_ref);
