@@ -27,6 +27,12 @@
 --
 --
 -- $Log: prep_disk.c,v $
+-- Revision 1.62  2008/09/06 15:34:23  jsedwards
+-- Changed to use TEST_ENVIRONMENT_VARIABLE.
+--
+-- Revision 1.61  2008/09/06 13:36:50  jsedwards
+-- Change pass phrase from TEST_PASS_PHRASE define to environment variable.
+--
 -- Revision 1.60  2008/09/01 18:24:14  jsedwards
 -- Moved almost all of the code that prepared to create the root object into
 -- the nwos_create_root function in objectify.c.
@@ -701,9 +707,10 @@ int main(int argc, char* argv[])
 	}
     }
 
-#ifndef TEST_PASS_PHRASE
-    agree_no_warranty();   /* make sure he or she agrees with the licensing */
-#endif
+    if (getenv(TEST_ENVIRONMENT_VARIABLE) == NULL)
+    {
+	agree_no_warranty();   /* make sure he or she agrees with the licensing */
+    }
 
     reserved_private_blocks = total_blocks - BLOCKS_IN_CHUNK;  /* first chunk is for header */
 
@@ -862,9 +869,11 @@ int main(int argc, char* argv[])
 
     if (do_private_blocks)
     {
-#ifndef TEST_PASS_PHRASE
-	understand_security();   /* make sure he or she understands the risks */
-#endif
+	if (getenv(TEST_ENVIRONMENT_VARIABLE) == NULL)
+	{
+	    understand_security();   /* make sure he or she understands the risks */
+	}
+
 	nwos_create_root();
     }
 
